CREATE OR REPLACE PACKAGE sp_snapsys
AS

	/**************************************************************************************************************
	*
	*  Project     : Systems Maintenance
	*
	*  Versions		 : 4.1
	*
	*  Created By  : George Leonard, george@mighty.co.za (always reachable here)
	*  Date        : 15/07/2006
	*
	*  Oracle Vers : Has been tested against 9.2.0.4
	*
	*  Description : Record system performance for painting forms and pl/sql web pages
	*
	*	 Dependency	 : snapsys_errors, snapsys_sysnames, snapsys_objnames, snapsys_logs, snapsys_objects, snapsys_avgs
	*       			 :
	*
	*  Modified on : 03/02/2005	added 2nd snaplog overloaded version on d_start as varchar2
	*                             moved code from snaplog to log procedure, called from either of the 2 snaplog proceudures
	*                             added getdate function to return a timestamp as a string. required for use in forms 6 that
	* 													  can not handle timestamp
	*              : 15/07/2006  extract object create script tocreobjs.sql, remember to keep the commented script at bottom of
	*                            this file in line with creobjs.sql
	*              : 15/09/2006  (3.0) added freedescription
	*              : 22/11/2006  (4.0) change behaviour of 'L' mode, it will now record to snapsys_logs and insert new record into snapsys_objnames
	*							 : 24/11/2006  (4.1) add user name return function: getusername, added dbms_application.read_module and set_module calls
	*                                  renamed snapsys.log to snapsys.slog (the log conflicted internally potentially with oracle reserved word/sqldeveloper error/warning).
	*
	***************************************************************************************************************/

	-- Get username of calling user
	 FUNCTION GETUSERNAME RETURN VARCHAR2;


	-- Get system mode for system/object being queried, if active then the objectid (snapsys_objnames.seq_num) is returned
   FUNCTION GETMODE (
		v_system		IN		VARCHAR2,
		v_object		IN		VARCHAR2
   ) RETURN NUMBER;

	 -- returns a the systimestamp as a varchar string, used by forms that does not support systimestamp variable type.
   FUNCTION GETDATE RETURN VARCHAR2;

	-- We are using overloading to be able to take the timestamp/d_start as a date formated varchar2 string or as a timestamp value
	PROCEDURE SNAPLOG (
	  v_system		IN 		VARCHAR2,
		v_objname		IN 		VARCHAR2,
		v_username	IN		VARCHAR2,
		d_start			IN		TIMESTAMP 	DEFAULT systimestamp,
		v_notes			IN		VARCHAR2	 	DEFAULT ''
 	);

	PROCEDURE SNAPLOG (
	  v_system		IN 		VARCHAR2,
		v_objname		IN 		VARCHAR2,
		v_username	IN		VARCHAR2,
		v_start			IN		VARCHAR2,
		v_notes			IN		VARCHAR2		DEFAULT ''
 	);

	-- Write record to log file
	PROCEDURE SLOG (
	  v_system		IN 		VARCHAR2,
		v_objname		IN 		VARCHAR2,
		v_username	IN		VARCHAR2,
		d_start			IN		TIMESTAMP 	DEFAULT systimestamp,
		v_notes			IN		VARCHAR2	 	DEFAULT ''
 	);

END sp_snapsys;
/

CREATE OR REPLACE PACKAGE BODY sp_snapsys
AS

	/**************************************************************************************************
	*
	* FUNCTION : getusername
	*
	* DEPENDENCIES:
	*
	****************************************************************************************************/
	FUNCTION getusername RETURN VARCHAR2 IS

  	v_error         varchar2(100);
		v_errorNum      varchar2(10);
		v_username			varchar2(30);

		v_save_module		varchar2(48);
		v_save_action		varchar2(32);

	BEGIN

		dbms_application_info.read_module(v_save_module, v_save_action);
		dbms_application_info.set_module('SNAPSYS.getusername', 'Starting');

		SELECT
			distinct(b.username)
		INTO
			v_username
		FROM v$session b
		WHERE b.sid = (SELECT DISTINCT(sid) FROM v$mystat);

		dbms_application_info.set_module(v_save_module, v_save_action);

		return v_username;

	EXCEPTION
	    WHEN OTHERS THEN
		   /* Catch any Oracle Errors */
      v_error 		:= SUBSTR(SQLERRM, 12, 100);
			v_errorNum 	:= SUBSTR(SQLCODE, 1, 5);
			INSERT INTO snapsys_errors (msg_desc) VALUES ( 'SP_SNAPSYS.GETUSERNAME: ORA:' || v_errorNum || ' DESCRIPTION:'|| v_error );
			COMMIT;
		  dbms_application_info.set_module(v_save_module, v_save_action);

	END getusername;


	/**************************************************************************************************
	*
	* FUNCTION : getdate
	*
	* DEPENDENCIES:
	*
	****************************************************************************************************/
	FUNCTION getdate  RETURN VARCHAR2 IS

	BEGIN

		return to_char(systimestamp,'DD/MM/YYYY HH24:MI:SS.FF4');

	END getdate;


	/**************************************************************************************************
	*
	* FUNCTION : getmode
	*
	* DEPENDENCIES: snapsys_objnames, snapsys_sysnames, snapsys_errors
	*
	****************************************************************************************************/
	FUNCTION getmode (
	   v_System    	IN   	VARCHAR2,
		 v_object			IN		VARCHAR2
	) RETURN NUMBER
	IS

  	v_error   		varchar2(100);
		v_errorNum    varchar2(10);

		n_objid				number 			:= 0;
		l_active_ind	varchar2(1) := 'N';

		v_save_module		varchar2(48);
		v_save_action		varchar2(32);

	BEGIN

		dbms_application_info.read_module(v_save_module, v_save_action);
		dbms_application_info.set_module('SNAPSYS.getmode', 'Starting');

		BEGIN
			SELECT
				a.seq_num
				,a.active_ind
			INTO
				n_objid
				,l_active_ind
			FROM snapsys_objnames a, snapsys_sysnames b
			WHERE	a.sysseq_num = b.seq_num
				and a.name = v_object
				and b.name = v_system
				and (b.active_ind = 'R' or b.active_ind = 'L');

		EXCEPTION
	 		WHEN no_data_found THEN
				n_objid 			:= 0;
				l_active_ind  := 'N';

	  	END;

		-- l_active_ind= 'Y' implies snapsys_sysnames.active_ind = R (run aka logging mode) AND snapsys_objnames.active_ind = Y
		IF (l_active_ind = 'Y') THEN
			return n_objid;

		ELSE
			return 0;

		END IF;
		dbms_application_info.set_module(v_save_module, v_save_action);

	EXCEPTION
	    WHEN OTHERS THEN
		   /* Catch any Oracle Errors */
      v_error 		:= SUBSTR(SQLERRM, 12, 100);
			v_errorNum 	:= SUBSTR(SQLCODE, 1, 5);
			INSERT INTO snapsys_errors (msg_desc) VALUES ( 'SP_SNAPSYS.GETMODE: ORA:' || v_errorNum || ' DESCRIPTION:'|| v_error );
			COMMIT;
		  dbms_application_info.set_module(v_save_module, v_save_action);

	END getmode;

	/**************************************************************************************************
	*
	* Procedure : snaplog
	*
	* DEPENDENCIES: snapsys_logs, snapsys_objects, snapsys_errors
	*
	****************************************************************************************************/
	PROCEDURE snaplog (
		v_system		IN 		VARCHAR2,
		v_objname		IN 		VARCHAR2,
		v_username	IN		VARCHAR2,
		d_start			IN		TIMESTAMP,
		v_notes			IN		VARCHAR2		DEFAULT ''
	)
	IS

		v_save_module		varchar2(48);
		v_save_action		varchar2(32);

	BEGIN

		dbms_application_info.read_module(v_save_module, v_save_action);
		dbms_application_info.set_module('SNAPSYS.snaplog1', 'Starting');

		SLOG(v_system, v_objname, v_username, d_start, v_notes);

		dbms_application_info.set_module(v_save_module, v_save_action);

	END snaplog;

	/**************************************************************************************************
	*
	* Procedure : snaplog
	*
	* DEPENDENCIES: snapsys_logs, snapsys_objects, snapsys_errors
	*
	****************************************************************************************************/
	PROCEDURE snaplog (
		v_system		IN 		VARCHAR2,
		v_objname		IN 		VARCHAR2,
		v_username	IN		VARCHAR2,
		v_start			IN		VARCHAR2,
		v_notes			IN		VARCHAR2		DEFAULT ''
	)
	IS

		d_start timestamp;

		v_save_module		varchar2(48);
		v_save_action		varchar2(32);

	BEGIN

		dbms_application_info.read_module(v_save_module, v_save_action);
		dbms_application_info.set_module('SNAPSYS.snaplog2', 'Starting');

		d_start := to_timestamp(v_start, 'DD/MM/YYYY HH24:MI:SS.FF4');

		SLOG(v_system, v_objname, v_username, d_start, v_notes);

		dbms_application_info.set_module(v_save_module, v_save_action);

	END snaplog;


	/**************************************************************************************************
	*
	* Procedure : slog
	*
	* DEPENDENCIES: snapsys_logs, snapsys_objects, snapsys_errors
	*
	****************************************************************************************************/
	PROCEDURE slog (
		v_system		IN 		VARCHAR2,
		v_objname		IN 		VARCHAR2,
		v_username	IN		VARCHAR2,
		d_start			IN		TIMESTAMP,
		v_notes			IN		VARCHAR2	DEFAULT ''
	)
	IS pragma autonomous_transaction;

  	v_error          	varchar2(100);
		v_errorNum       	varchar2(10);

		n_sysseq_num			number			:= 0;
		n_objid						number 			:= 0;
		l_active_ind			varchar2(1) := 'N';

		v_save_module		varchar2(48);
		v_save_action		varchar2(32);

	BEGIN

		dbms_application_info.read_module(v_save_module, v_save_action);
		dbms_application_info.set_module('SNAPSYS.slog', 'Starting');

		-- Find if system given is in On/Run mode, learn mode or off
		BEGIN
			SELECT
				a.active_ind,
				a.seq_num
			INTO
				l_active_ind,
				n_sysseq_num
			FROM snapsys_sysnames a
			WHERE	a.name = v_system;

		EXCEPTION
	 		WHEN no_data_found THEN
				l_active_ind := 'N';

	  	END;

		n_objid := getmode(v_system, v_objname);
		IF (l_active_ind = 'L' and n_objid = 0) THEN
			INSERT INTO snapsys_objects(system_name, object_name) VALUES (v_system, v_objname);
			INSERT INTO snapsys_objnames (sysseq_num, name, active_ind) VALUES (n_sysseq_num, v_objname,'Y');
			COMMIT;
			n_objid := getmode(v_system, v_objname);

		END IF;

		IF (n_objid > 0) THEN
			INSERT INTO snapsys_logs (objid, username, call_time, fin_time, notes) VALUES ( n_objid, decode(v_username,'',getusername(),v_username), d_start, SYSTIMESTAMP, trim(substr(v_notes,1, 999)));
			COMMIT;

		END IF;
		dbms_application_info.set_module(v_save_module, v_save_action);

	EXCEPTION
	    WHEN OTHERS THEN
		   /* Catch any Oracle Errors */
      v_error 		:= SUBSTR(SQLERRM, 12, 100);
			v_errorNum 	:= SUBSTR(SQLCODE, 1, 5);
			INSERT INTO snapsys_errors (msg_desc) VALUES ( 'SP_SNAPSYS.SLOG: ORA:' || v_errorNum || ' DESCRIPTION:'|| v_error );
			COMMIT;
		  dbms_application_info.set_module(v_save_module, v_save_action);

	END slog;



	/**************************************************************************************************************
	*
	*  INSTALLATIONS:
	*  	Dependency: Tablespace audit_small_data, audit_small_indexes
	*
	*********************************************************************************************
	*
	*
	*	CREATE TABLE snapsys_sysnames (
	*	   seq_num					number,
	*     name						varchar2(32),
	*     active_ind      varchar2(1)	default 'N',
	*     diag_ind			 	varchar2(1) default 'N',
	*     freedescription varchar2(128)
	*	) tablespace audit_small_data;
	*
	*  COMMENT ON TABLE snapsys_sysnames IS 'Different System names, grouping of systems running in the cor schema IE ACQ/SMAC/SPIF'
	*  /
	*  COMMENT ON COLUMN snapsys_sysnames.seq_num IS 'Sequential numbering'
	*  /
	*  COMMENT ON COLUMN snapsys_sysnames.name IS 'Short Name/Abbreviation'
	*  /
	*  COMMENT ON COLUMN snapsys_sysnames.active_ind IS 'On/Enabled=>R, Learn Mode=>L, Off=>N'
	*  /
	*  COMMENT ON COLUMN snapsys_sysnames.diag_ind IS 'system diagnostic tracing via dbms_support or dbms_profiler, On=>Y, Off/N'
	*  /
	*
	*  create unique index sssn_pk on snapsys_sysnames(seq_num) tablespace audit_small_indexes;
	*	create unique index sssn_uk1 on snapsys_sysnames(name) tablespace audit_small_indexes;
	*	create index sssn_idx1 on snapsys_sysnames(name, active_ind, seq_num) tablespace audit_small_indexes;
	*
	*	PROMPT Creating Primary Key on 'snapsys_sysnames'
	*	ALTER TABLE snapsys_sysnames
	*	 ADD (CONSTRAINT sssn_PK PRIMARY KEY
	*	  (seq_num))
	*	/
	*
	*	PROMPT Creating Check Constraint on 'snapsys_sysnames'
	*	ALTER TABLE snapsys_sysnames
	*	 ADD (CONSTRAINT SSSN_CKY_ACTIVE_IND CHECK (active_ind IN ('R', 'L', 'N')))
	*	/
	*
	*  PROMPT Creating Sequence 'SSSN_SEQ_NUM'
	*	CREATE SEQUENCE SSSN_SEQ_NUM
	*	 NOMAXVALUE
	*	 NOMINVALUE
	*	 NOCYCLE
	*	 NOCACHE
	*	/
	*
	*
	*	PROMPT Creating Trigger 'SSSN_BIR_SEQ_NUM'
	*	CREATE OR REPLACE TRIGGER SSSN_BIR_SEQ_NUM
	*	 BEFORE INSERT
	*	 ON snapsys_SYSNAMES
	*	 FOR EACH ROW
	*	 WHEN (NEW.seq_num IS NULL)
	*	DECLARE
	*	-- 	Author      : George Leonard
	*	--   	Date        : 27 Janurary 2005
	*	--   	Description : This trigger gets the next value from SSSN_SEQ_NUM sequence before INSERT
	*
	*	   v_retmesg         VARCHAR2(2000);
	*
	*	BEGIN
	*	   SELECT SSSN_SEQ_NUM.NEXTVAL
	*	     INTO :NEW.SEQ_NUM
	*	     FROM DUAL;
	*	EXCEPTION
	*	   WHEN OTHERS THEN
	*			NULL;
	*	END;
	*	/
	*  show err
	*
	*  CREATE TABLE snapsys_objnames (
	*     seq_num						number,
	*     sysseq_num				number,
	*     name							varchar2(128),
	*     active_ind        varchar2(1) default 'N',
	*     diag_ind					number default 0,
	*     freedescription  	varchar2(128)
	*	) tablespace audit_small_data;
	*
	*  COMMENT ON TABLE snapsys_objnames IS 'Different Object Names to be monitored, IE Different Form names or different Stored Procedures, Packages or functions or Triggers'
	*  /
	*  COMMENT ON COLUMN snapsys_objnames.seq_num IS 'Sequential numbering'
	*  /
	*  COMMENT ON COLUMN snapsys_objnames.seq_num IS 'System ID (snapsys_sysnames.seq_num) to which this object belongs'
	*  /
	*  COMMENT ON COLUMN snapsys_objnames.name IS 'Object identifyer, IE Procedure name or package.procedure name or trigger name'
	*  /
	*  COMMENT ON COLUMN snapsys_objnames.active_ind IS 'On/Enabled=>Y, Off=>N for timing logging'
	*  /
	*  COMMENT ON COLUMN snapsys_objnames.diag_ind IS '1 for dbms_profiler start or 100466, 100468, 1004612, 100530 for tkprof tracing'
	*  /
	*
	*  create unique index sson_pk on snapsys_objnames(seq_num) tablespace audit_small_indexes;
	*  create unique index sson_uk1 on snapsys_objnames(sysseq_num, name) tablespace audit_small_indexes;
	*
	*	PROMPT Creating Primary Key on 'snapsys_objnames'
	*	ALTER TABLE snapsys_objnames
	*	 ADD (CONSTRAINT sson_PK PRIMARY KEY
	*	  (seq_num))
	*	/
	*
	*	ALTER TABLE snapsys_objnames ADD (
	*	  CONSTRAINT SSON_SEQ_NUM_FK FOREIGN KEY (SYSSEQ_NUM)
	*	    REFERENCES snapsys_SYSNAMES (SEQ_NUM));
	*
	*	PROMPT Creating Check Constraint on 'snapsys_objnames'
	*	ALTER TABLE snapsys_objnames
	*	 ADD (CONSTRAINT SSON_CKY_ACTIVE_IND CHECK (active_ind IN ('Y', 'N')))
	*	/
	*
	*	PROMPT Creating Sequence 'SSON_SEQ_NUM'
	*	CREATE SEQUENCE SSON_SEQ_NUM
	*	 NOMAXVALUE
	*	 NOMINVALUE
	*	 NOCYCLE
	*	 NOCACHE
	*	/
	*
	*	PROMPT Creating Trigger 'SSON_BIR_SEQ_NUM'
	*	CREATE OR REPLACE TRIGGER SSON_BIR_SEQ_NUM
	*	 BEFORE INSERT
	*	 ON snapsys_OBJNAMES
	*	 FOR EACH ROW
	*	 WHEN (NEW.seq_num IS NULL)
	*	DECLARE
	*	-- 	Author      : George Leonard
	*	--   	Date        : 27 Janurary 2005
	*	--   	Description : This trigger gets the next value from SSON_SEQ_NUM sequence before INSERT
	*
	*	   v_retmesg         VARCHAR2(2000);
	*
	*	BEGIN
	*	   SELECT SSON_SEQ_NUM.NEXTVAL
	*	     INTO :NEW.SEQ_NUM
	*	     FROM DUAL;
	*	EXCEPTION
	*	   WHEN OTHERS THEN
	*			NULL;
	*	END;
	*	/
	*
	*  -- This table is going to fast and get BIG!!! maybe have a bigger extend (5m) based tablespace for it and associated indexes
	*
	*	CREATE TABLE snapsys_logs (
	*     objid							number,
	*     username					varchar2(32),
  *     call_time       	timestamp default systimestamp,
  *     fin_time       		timestamp default systimestamp,
	*			notes							varchar2(1000)
	*	) tablespace audit_small_data;
	*
	*  COMMENT ON TABLE snapsys_logs IS 'Records of for every time the object designated was called'
	*  /
	*  COMMENT ON COLUMN snapsys_logs.objid IS 'Seq_num from snapsys_objnames for object'
	*  /
	*  COMMENT ON COLUMN snapsys_logs.username IS 'User name executing the call, W Number'
	*  /
	*  COMMENT ON COLUMN snapsys_logs.call_time IS 'Call Start, Start of Form Paint or Procedure/Package/Trigger Start'
	*  /
	*  COMMENT ON COLUMN snapsys_logs.fin_time IS 'Call End, End of Form Paint or Procedure/Package/Trigger Start, The Delta to call_time is the job execution time'
	*  /
	*  COMMENT ON COLUMN snapsys_logs.notes IS 'Free Form Note'
	*  /
	*
	*  create index slog_idx1 on snapsys_logs(objid, call_time, username) tablespace audit_small_indexes;
	*
	*
	*	CREATE TABLE snapsys_errors (
 	*     msg_date       		timestamp default systimestamp,
 	*	   	msg_desc      		varchar2(1000)
	*	) tablespace audit_small_data;
	*
	*  COMMENT ON TABLE snapsys_errors IS 'Output Catch for "Exception / When Others"'
	*  /
	*  COMMENT ON COLUMN snapsys_errors.msg_date IS 'Systimestamp when error occured'
	*  /
	*  COMMENT ON COLUMN snapsys_errors.msg_desc IS 'Ora Error number and description'
	*  /
	*
	*	 CREATE TABLE snapsys_objects (
 	*    system_name      	varchar2(32),
 	*	   object_name				varchar2(256)
	*	 ) tablespace audit_small_data;
	*
	*  COMMENT ON TABLE snapsys_objects IS 'This table gets populated when a system is marked as Learn mode in snapsys_sysnames, This information can then be used to populate snapsys_objnames'
	*  /
	*  COMMENT ON COLUMN snapsys_objects.system_name IS 'System Name'
	*  /
	*  COMMENT ON COLUMN snapsys_objects.object_name IS 'Calling Object identifier'
	*  /
	*
	*	 CREATE TABLE snapsys_avgs (
	*     objid							number,
	*     hourofday					varchar2(2),
	* 	  min_time					number(8,4),
  *     avg_time       		number(8,4),
	* 	  max_time					number(8,4),
	*     log_date					date default SYSDATE,
	* 	  notes							varchar2(1000)
	*	) tablespace audit_small_data;
	*
	*  COMMENT ON TABLE snapsys_avgs IS 'This will be the base line values that is used to determine if object is running faster or slower than normal'
	*  /
	*  COMMENT ON COLUMN snapsys_avgs.objid IS 'seq_num from snapsys_objnames for object'
	*  /
	*  COMMENT ON COLUMN snapsys_avgs.hourofday IS 'If we care grouping baseline values per hour of day, the hour of the day the data is applicable to'
	*  /
	*  COMMENT ON COLUMN snapsys_avgs.min_time IS 'Min duration for object call'
	*  /
	*  COMMENT ON COLUMN snapsys_avgs.avg_time IS 'Avg duration for object call'
	*  /
	*  COMMENT ON COLUMN snapsys_avgs.max_time IS 'Max duration for object call'
	*  /
	*  COMMENT ON COLUMN snapsys_avgs.log_date IS 'Date baseline value was stored'
	*  /
	*  COMMENT ON COLUMN snapsys_avgs.notes IS 'User comment for baseline value'
	*  /
	*
	*  create index ssav_idx1 on snapsys_avgs(objid) tablespace audit_small_indexes;
	*  create index ssav_idx2 on snapsys_avgs(objid, hourofday) tablespace audit_small_indexes;
	*
	*	ALTER TABLE snapsys_avgs ADD (
	*	  CONSTRAINT SSAV_SEQ_NUM_FK FOREIGN KEY (OBJID)
	*	    REFERENCES snapsys_OBJNAMES (seq_num));
	*
	* !! Create Package
	*
	* -- create public synonym sp_snapsys for snaps.sp_snapsys;
	* -- grant execute on snaps.sp_snapsys to public;
	*
	***************************************************************************************************************/

END sp_snapsys;
/
